%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                                                                       %
%   In this M-file one can introduce parameters into the model,         %
%   and set the initial conditions and the simulation time.             %
%                                                                       %
%                                                                       %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    %                                                           %
    %                                                           %
    %         Mebrane complexes                                 %
    %                                                           %  
    %         y(1)   pMHC1   free                               %    
    %         y(2)   TCR|pMHC1                                  %
    %         y(3)   LCK|TCR|pMHC1                              %
    %         y(4)   LCKs|TCR|pMHC1                             %
    %         y(5)   LCKy|TCR|pMHC1                             %
    %         y(6)   LCKsy|TCR|pMHC1                            %
    %         y(7)   LCKy|pTCR|pMHC1                            %
    %         y(8)   LCKsy|pTCR|pMHC1                           %
    %         y(9)   LCKy|ppTCR|pMHC1                           %
    %         y(10)  LCKsy|ppTCR|pMHC1                          %
    %                                                           %
    %         y(11)   pMHC2  free                               %    
    %         y(12)   TCR|pMHC2                                 %
    %         y(13)   LCK|TCR|pMHC2                             %
    %         y(14)   LCKs|TCR|pMHC2                            %
    %         y(15)   LCKy|TCR|pMHC2                            %
    %         y(16)   LCKsy|TCR|pMHC2                           %
    %         y(17)   LCKy|pTCR|pMHC2                           %
    %         y(18)   LCKsy|pTCR|pMHC2                          %
    %         y(19)   LCKy|ppTCR|pMHC2                          %
    %         y(20)   LCKsy|ppTCR|pMHC2                         %
    %                                                           %
    %         y(21)  TCR free                                   %
    %                                                           % 
    %         y(22)       pSHP|TCR                              %
    %         y(23)       pSHP|TCR|pMHC1                        % 
    %         y(24)       pSHP|LCK|TCR|pMHC1 complex            %
    %         y(25)       pSHP|TCR|pMHC2                        %
    %         y(26)       pSHP|LCK|TCR|pMHC2                    %
    %                                                           %
    %         Cytosolic proteins                                %
    %                                                           %
    %         y(27)   LCK                                       %
    %         y(28)   SHP free                                  %
    %         y(29)   pSHP free                                 %
    %         y(30)   ZAP                                       %
    %         y(31)   ZAPp                                      %
    %         y(32)   MEK                                       %
    %         y(33)   pMEK                                      %
    %         y(34)   ppMEK                                     %
    %         y(35)   ERK                                       %
    %         y(36)   pERK                                      %
    %         y(37)   ppERK                                     %
    %                                                           %
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

for ttti=1:round(sum(clock))     % randomization of randomization
    rand;
end;    
    rand;

%############## Simulation time points ##################################
%########################################################################


t0=1000;             % equlibration time (at time t0 one may add new peptides)
tw=1000;             % length of simulation from adding new peptides
ttot=t0+tw;          % total simulation time in seconds

%########################################################################
%############### Initial conditions  ####################################
%########################################################################

NC=37;    % number of substrates
NR=87;    % number of reactions (not 97 as misprinted in the paper)

y0=zeros(1,NC); 
y=zeros(1,NC);
yy=zeros(ttot+1,NC);  

T=zeros(ttot+1,1);

% p=zeros(NC);          %propensities of one component reaction
% m=zeros(NC);          %propensities of one component reaction
% pm=zeros(NC,NC);      %propensities of two component reaction
% pmm=zeros(NC,NC,NC);  %propensities of three component reaction (synthesis)
% mpp=zeros(NC,NC,NC);  %propensities of three component reaction (dissociation)
% mppp=zeros(NC,NC,NC); %propensities of four component reaction (dissociation)

prop=zeros(NR,1);        %propensities
v=zeros(NR,NC);          %reaction vectors

%%%%  change this initial conditions to test bistability of the system %%%%
%%%%  the below values are from deterministic simulation with no peptide stimulation -resting cells %%%% 

y0(10)=0;              %LCKsy|ppTCR|pMHC1
y0(20)=0;              %LCKsy|ppTCR|pMHC2
y0(31)=10;              %pZAP 
y0(33)=4760;            %4760 pMEK 
y0(34)=60;              %60 ppMEK 
y0(36)=27000;           %27000 pERK
y0(37)=670;             %670  ppERK   
y0(29)=500;             %500 pSHP free
y0(22)=4400;            %4400 TCR|pSHP 

y=y0;


%%%%%%%%% Other initial conditions %%%%%%%%%%%%

pMHC1=100;  % number of agonist peptide MHC added at t=0
pMHC11=0;   % number of agonist peptide MHC added at time t0
pMHC2=0;    % number of endogenous or antagonist peptide MHC added at t=0
pMHC22=300; % number of endogenous or antagonist peptide MHC added at time t0

% You can set pMHC1=0, pMHC11=100, pMHC2=300, pMHC22=0, i.e. add antagonist
% peptides first to test the bistability of the system % 

TCR=30000;
LCK=100000;
ZAP=100000;
MEK=100000;
ERK=300000;
SHP=300000; 


y0(1)=pMHC1;
y0(11)=pMHC2;
y0(21)=TCR-y0(22);
y0(27)=LCK;
y0(28)=SHP-y0(29)-y0(22);
y0(30)=ZAP-y(31);
y0(32)=MEK-y0(33)-y0(34);
y0(35)=ERK-y0(36)-y0(37);

y=y0;
yy(1,:)=y;
    
 
sp=10; % speed-up factor for fast reactions
% Fast reactions: 6, 37, 38, 39, 40, 45, 56, 61

%############### Parametrization for system of SE ####################
%########################################################################


%XXXXXXXXXXXXXXXXX Complex  XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX                                 

 b1=0.3/TCR;                 %0.3/TCR%  agonist peptide binding 
 b2=b1;                      %antagonist peptide binding 

 
 d1=0.05;                      %0.05 agonist  peptide disociation
 d2=0.333;                     %0.333 antagonist or endogenous peptide disociation
 
 lb=0.3/LCK;                %0.3/LCK%  LCk(s)  binding
 
 ly2=0.3;                   %0.3%    LCK(s) ->LCK(s)y  theorine phosporylation at complex
 ly1=5/SHP;                 %5/SHP  pSHP complex binding
 
 
 ls2=0.5/ERK;               %0.5/ERK         LCK(y) ->LCK s(y)     ppERK catalyzed serine phosphorylation
 ls1=0.1;                   %0.1             LCK (y)s ->LCK (y)    "spontaneuos" serine dephosphorylation
 
 tp=0.05;                   %0.05            TCR ->pTCR and pTCR->ppTCR 
 
 
 %XXXXXXXXXXXXXXXXXXX Cytosolic Proteins XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
 
 s0=10^(-5);                  %10^(-5)   SHP -> pSHP              SHP spontaneous phosphorylation
 s1=30/SHP;                   %30/SHP    SHP -> pSHP              SHP phosphorylation
 s2=0.0006;                   %0.0006    pSHP -> SHP              SHP dephosphorylation (results in immediate dissociation)
 s3=0.05;                     %0.05%                              SHP dissociation
 
 z0=2*10^(-6);                %2*10^(-6)   ZAP -> pZAP            ZAP spontaneous phosphorylation 
 z1=5/ZAP;                    %5/ZAP       ZAP -> pZAP            ZAP phosphorylation (catalyzed by ppTCR)
 z2=0.02;                     %0.02        pZAP -> ZAP            ZAP dephosphorylation 
 
 m1=5/MEK;                   %5/MEK    MEK -> pMEK and pMEK -> ppMEK           MEK phosphorylation (catalyzed by pZAP)
 m2=0.02;                    %0.02     ppMEK -> pMEK and pMEK -> MEK           MEK dephosphorylation
 
 e1=5/ERK;                   %5/ERK    ERK -> pERK and pERK -> ppERK           ERK phosphorylation (catalyzed by ppMEK)
 e2=0.02;                    %0.02     ppERK -> pERK  and pERK -> ERK          ERK dephosphorylation
 
 
 
 %#########################################################################
 %%%%%%%%%%%%%%% definition of reaction vectors %%%%%%%%%%%%%%%%%%%%%%%%%%%
 %#########################################################################
 %                v(reaction number, variable to be chanched)             %
               

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%% Two component channels pm %%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


%pm(31,30)
%prop(1)=(z0+z1*(y(9)+y(10)+y(19)+y(20)))*y(30);
v(1,31)=1; v(1,30)=-1;

%pm(30,31)
%prop(2)=z2*y(31);
v(2,30)=1; v(2,31)=-1;

%pm(33,32)
%prop(3)=2*m1*y(31)*y(32);
v(3,33)=1; v(3,32)=-1;

%pm(32,33)   
%prop(4)=m2*y(33);
v(4,32)=1, v(4,33)=-1;

%pm(36,35)
%prop(5)=2*e1*y(34)*y(35);
v(5,36)=1; v(5,35)=-1;

%pm(35,36)  
%prop(6)=e2*y(36)/sp; 
v(6,35)=1*sp;  v(6,36)=-1*sp;                                     %FAST

 

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%pm(5,3)                          
%prop(7)=ly2*y(3);                 
v(7,5)=1; v(7,3)=-1;

%pm(6,4)
%prop(8)=ly2*y(4);
v(8,6)=1; v(8,4)=-1;

%pm(15,13)
%prop(9)=ly2*y(13);
v(9,15)=1; v(9,13)=-1;

%pm(16,14)
%prop(10)=ly2*y(14);
v(10,16)=1; v(10,14)=-1;

%pm(7,5)
%prop(11)=tp*y(5);
v(11,7)=1; v(11,5)=-1;

%pm(8,6)
%prop(12)=tp*y(6);
v(12,8)=1; v(12,6)=-1;

%pm(9,7)
%prop(13)=tp*y(7);
v(13,9)=1; v(13,7)=-1;

%pm(10,8)
%prop(14)=tp*y(8);
v(14,10)=1; v(14,8)=-1;

%pm(17,15)
%prop(15)=tp*y(15);
v(15,17)=1; v(15,15)=-1;

%pm(18,16)
%prop(16)=tp*y(16);
v(16,18)=1; v(16,16)=-1;

%pm(19,17)
%prop(17)=tp*y(17);
v(17,19)=1; v(17,17)=-1;

%pm(20,18)
%prop(18)=tp*y(18);
v(18,20)=1; v(18,18)=-1;

%pm(3,4)
%prop(19)=ls1*y(4);
v(19,3)=1; v(19,4)=-1;

%pm(5,6)
%prop(20)=ls1*y(6);
v(20,5)=1; v(20,6)=-1;

%pm(7,8)
%prop(21)=ls1*y(8);
v(21,7)=1; v(21,8)=-1;

%pm(9,10)
%prop(22)=ls1*y(10);
v(22,9)=1; v(22,10)=-1;

%pm(13,14)
%prop(23)=ls1*y(14);
v(23,13)=1; v(23,14)=-1;

%pm(15,16)
%prop(24)=ls1*y(16);
v(24,15)=1; v(24,16)=-1;

%pm(17,18)
%prop(25)=ls1*y(18);
v(25,17)=1; v(25,18)=-1;

%pm(19,20)
%prop(26)=ls1*y(20);
v(26,19)=1; v(26,20)=-1;

%pm(4,3)
%prop(27)=ls2*y(37)*y(3);
v(27,4)=1; v(27,3)=-1;

%pm(6,5)
%prop(28)=ls2*y(37)*y(5);
v(28,6)=1; v(28,5)=-1;

%pm(8,7)
%prop(29)=ls2*y(37)*y(7);
v(29,8)=1; v(29,7)=-1;

%pm(10,9)
%prop(30)=ls2*y(37)*y(9);
v(30,10)=1; v(30,9)=-1;

%pm(14,13)
%prop(31)=ls2*y(37)*y(13);
v(31,14)=1; v(31,13)=-1;

%pm(16,15)
%prop(32)=ls2*y(37)*y(15);
v(32,16)=1; v(32,15)=-1;

%pm(18,17)
%prop(33)=ls2*y(37)*y(17);
v(33,18)=1; v(33,17)=-1;

%pm(20,19)
%prop(34)=ls2*y(37)*y(19);
v(34,20)=1; v(34,19)=-1;

%pm(34,33)
%prop(35)=m1*y(31)*y(33);
v(35,34)=1; v(35,33)=-1;

%pm(33,34)
%prop(36)=2*m2*y(34);
v(36,33)=1; v(36,34)=-1;

%pm(37,36)
%prop(37)=e1*y(34)*y(36);
v(37,37)=1*sp; v(37,36)=-1*sp;                                % FAST

%pm(36,37)
%prop(38)=2*e2*y(37);
v(38,36)=1*sp; v(38,37)=-1*sp;                                % FAST
      
%pm(29,28)
%prop(39)=s1*(y(5)+y(7)+y(9)+y(15)+y(17)+y(19))*y(28);        % FAST
v(39,29)=1*sp; v(39,28)=-1*sp;

%pm(28,29)
%prop(40)=s2*y(29);
v(40,28)=1*sp; v(40,29)=-1*sp;                                 %FAST

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%% There component: synthesis pmm %%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%pmm(2,1,21)
%prop(41)=b1*y(1)*y(21);                       
v(41,2)=1; v(41,1)=-1; v(41,21)=-1;

%pmm(12,11,21)
%prop(42)=b2*y(11)*y(21);
v(42,12)=1; v(42,11)=-1; v(42,21)=-1;

%pmm(3,2,27)
%prop(43)=lb*y(27)*y(2);
v(43,3)=1; v(43,2)=-1; v(43,27)=-1;

%pmm(13,12,27)
%prop(44)=lb*y(27)*y(12);
v(44,13)=1; v(44,12)=-1; v(44,27)=-1;

%pmm(22,21,29)
%prop(45)=ly1*y(21)*y(29);
v(45,22)=1*sp; v(45,21)=-1*sp; v(45,29)=-1*sp;                  % FAST

%pmm(23,2,29)
%prop(46)=ly1*y(2)*y(29);
v(46,23)=1; v(46,2)=-1; v(46,29)=-1;

%pmm(24,3,29)
%prop(47)=ly1*y(3)*y(29);
v(47,24)=1; v(47,3)=-1; v(47,29)=-1;

%pmm(24,5,29)
%prop(48)=ly1*y(5)*y(29);
v(48,24)=1; v(48,5)=-1; v(48,29)=-1;

%pmm(24,7,29)
%prop(49)=ly1*y(7)*y(29);
v(49,24)=1; v(49,7)=-1; v(49,29)=-1;

%pmm(24,9,29)
%prop(50)=ly1*y(9)*y(29);
v(50,24)=1; v(50,9)=-1; v(50,29)=-1;

%pmm(25,12,29)
%prop(51)=ly1*y(12)*y(29);
v(51,25)=1; v(51,12)=-1; v(51,29)=-1;

%pmm(26,13,29)
%prop(52)=ly1*y(13)*y(29);
v(52,26)=1; v(52,13)=-1; v(52,29)=-1;

%pmm(26,15,29)
%prop(53)=ly1*y(15)*y(29);
v(53,26)=1; v(53,15)=-1; v(53,29)=-1;

%pmm(26,17,29)
%prop(54)=ly1*y(17)*y(29);
v(54,26)=1; v(54,17)=-1; v(54,29)=-1;

%pmm(26,19,29)
%prop(55)=ly1*y(19)*y(29);
v(55,26)=1; v(55,19)=-1; v(55,29)=-1;


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%% There component: dissociation mpp %%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%mpp(22,21,28)
%prop(56)=s2*y(22);                 
v(56,22)=-1*sp; v(56,21)=1*sp; v(56,28)=1*sp;                  % FAST 

%mpp(23,2,28)
%prop(57)=s2*y(23);
v(57,23)=-1; v(57,2)=1; v(57,28)=1;

%mpp(25,12,28)
%prop(58)=s2*y(25);
v(58,25)=-1; v(58,12)=1; v(58,28)=1;

%mpp(24,3,28)
%prop(59)=s2*y(24);
v(59,24)=-1; v(59,3)=1; v(59,28)=1;

%mpp(26,13,28)
%prop(60)=s2*y(26);
v(60,26)=-1; v(60,13)=1; v(60,28)=1;

%mpp(22,21,29)
%prop(61)=s3*y(22);
v(61,22)=-1*sp; v(61,21)=1*sp; v(61,29)=1*sp;                      %FAST

%mpp(23,2,29)
%prop(62)=s3*y(23);
v(62,23)=-1; v(62,2)=1; v(62,29)=1;

%mpp(25,12,29)
%prop(63)=s3*y(25);
v(63,25)=-1; v(63,12)=1; v(63,29)=1;

%mpp(24,3,29)
%prop(64)=s3*y(24);
v(64,24)=-1; v(64,3)=1; v(64,29)=1;

%mpp(26,13,29)
%prop(65)=s3*y(26);
v(65,26)=-1; v(65,13)=1; v(65,29)=1;

%mpp(2,1,21)
%prop(66)=d1*y(2);
v(66,2)=-1; v(66,1)=1; v(66,21)=1;

%mpp(12,11,21)
%prop(67)=d2*y(12);
v(67,12)=-1; v(67,11)=1; v(67,21)=1;

%mpp(23,1,22)
%prop(68)=d1*y(23);
v(68,23)=-1; v(68,1)=1; v(68,22)=1;

%mpp(25,11,22)
%prop(69)=d2*y(25);
v(69,25)=-1; v(69,11)=1; v(69,22)=1;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%% Four component dissociation %%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%mppp(3,1,21,27)
%prop(70)=d1*y(3);           
v(70,3)=-1; v(70,1)=1; v(70,21)=1; v(70,27)=1;

%mppp(4,1,21,27)
%prop(71)=d1*y(4);
v(71,4)=-1; v(71,1)=1; v(71,21)=1; v(71,27)=1;


%mppp(5,1,21,27)
%prop(72)=d1*y(5);
v(72,5)=-1; v(72,1)=1; v(72,21)=1; v(72,27)=1;

%mppp(6,1,21,27)
%prop(73)=d1*y(6);
v(73,6)=-1; v(73,1)=1; v(73,21)=1; v(73,27)=1;

%mppp(7,1,21,27)
%prop(74)=d1*y(7);
v(74,7)=-1; v(74,1)=1; v(74,21)=1; v(74,27)=1;


%mppp(8,1,21,27)
%prop(75)=d1*y(8);
v(75,8)=-1; v(75,1)=1; v(75,21)=1; v(75,27)=1;

%mppp(9,1,21,27)
%prop(76)=d1*y(9);
v(76,9)=-1; v(76,1)=1; v(76,21)=1; v(76,27)=1;

%mppp(10,1,21,27)
%prop(77)=d1*y(10);
v(77,10)=-1; v(77,1)=1; v(77,21)=1; v(77,27)=1;

%mppp(13,11,21,27)
%prop(78)=d2*y(13);
v(78,13)=-1; v(78,11)=1; v(78,21)=1; v(78,27)=1;

%mppp(14,11,21,27)
%prop(79)=d2*y(14);
v(79,14)=-1; v(79,11)=1; v(79,21)=1; v(79,27)=1;

%mppp(15,11,21,27)
%prop(80)=d2*y(15);
v(80,15)=-1; v(80,11)=1; v(80,21)=1; v(80,27)=1;

%mppp(16,11,21,27)
%prop(81)=d2*y(16);
v(81,16)=-1; v(81,11)=1; v(81,21)=1; v(81,27)=1;

%mppp(17,11,21,27)
%prop(82)=d2*y(17);
v(82,17)=-1; v(82,11)=1; v(82,21)=1; v(82,27)=1;

%mppp(18,11,21,27)
%prop(83)=d2*y(18);
v(83,18)=-1; v(83,11)=1; v(83,21)=1; v(83,27)=1;

%mppp(19,11,21,27)
%prop(84)=d2*y(19);
v(84,19)=-1; v(84,11)=1; v(84,21)=1; v(84,27)=1;

%mppp(20,11,21,27)
%prop(85)=d2*y(20);
v(85,20)=-1; v(85,11)=1; v(85,21)=1; v(85,27)=1;

%mppp(24,1,27,22)
%prop(86)=d1*y(24);
v(86,24)=-1; v(86,1)=1; v(86,27)=1; v(86,22)=1;

%mppp(26,11,27,22)
%prop(87)=d2*y(26);
v(87,26)=-1; v(87,11)=1; v(87,27)=1; v(87,22)=1;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%









 