%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                                                                      %
%   Code for solving Gillespie Algorithm for TCR system described in   %
%   T. Lipniacki, B. Hat, J.R. Faeder, W.S. Hlavacek                   %
%   Stochastic effects and bistability in T cell receptor signaling    %
%   J. Theor. Biol. 2008 doi 10.1016/j.jtbi.2008.05.001                %
%                                                                      %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    %                                                           %
    %                                                           %
    %         Mebrane complexes                                 %
    %                                                           %  
    %         y(1)   pMHC1   free                               %    
    %         y(2)   TCR|pMHC1                                  %
    %         y(3)   LCK|TCR|pMHC1                              %
    %         y(4)   LCKs|TCR|pMHC1                             %
    %         y(5)   LCKy|TCR|pMHC1                             %
    %         y(6)   LCKsy|TCR|pMHC1                            %
    %         y(7)   LCKy|pTCR|pMHC1                            %
    %         y(8)   LCKsy|pTCR|pMHC1                           %
    %         y(9)   LCKy|ppTCR|pMHC1                           %
    %         y(10)  LCKsy|ppTCR|pMHC1                          %
    %                                                           %
    %         y(11)   pMHC2  free                               %    
    %         y(12)   TCR|pMHC2                                 %
    %         y(13)   LCK|TCR|pMHC2                             %
    %         y(14)   LCKs|TCR|pMHC2                            %
    %         y(15)   LCKy|TCR|pMHC2                            %
    %         y(16)   LCKsy|TCR|pMHC2                           %
    %         y(17)   LCKy|pTCR|pMHC2                           %
    %         y(18)   LCKsy|pTCR|pMHC2                          %
    %         y(19)   LCKy|ppTCR|pMHC2                          %
    %         y(20)   LCKsy|ppTCR|pMHC2                         %
    %                                                           %
    %         y(21)  TCR free                                   %
    %                                                           % 
    %         y(22)       pSHP|TCR                              %
    %         y(23)       pSHP|TCR|pMHC1                        % 
    %         y(24)       pSHP|LCK|TCR|pMHC1 complex            %
    %         y(25)       pSHP|TCR|pMHC2                        %
    %         y(26)       pSHP|LCK|TCR|pMHC2                    %
    %                                                           %
    %         Cytosolic proteins                                %
    %                                                           %
    %         y(27)   LCK                                       %
    %         y(28)   SHP free                                  %
    %         y(29)   pSHP free                                 %
    %         y(30)   ZAP                                       %
    %         y(31)   ZAPp                                      %
    %         y(32)   MEK                                       %
    %         y(33)   pMEK                                      %
    %         y(34)   ppMEK                                     %
    %         y(35)   ERK                                       %
    %         y(36)   pERK                                      %
    %         y(37)   ppERK                                     %
    %                                                           %
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    
clear;
clc;
%%%%%%%%%%%%% Additional Parameters %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
parameters;            % parametrization
zz=zeros(ttot+1,NC);   % average from NNN cells 

NNN = input('How many cells?  ')  % number of cells


starttime=clock;       %curent time
nstep=0;               %number of steps

for ii=1:NNN           %beginning of the main loop

parameters;            % parametrization 


tbiol=0;                %  biological time

nn=1;                  %number of recoded steps

while (tbiol < ttot)    %beginning of single cell loop

%%%%%%%%%% PROPENSITIES %%%%%%%%%%%%%%%

% Fast reactions: 6, 37, 38, 39, 40, 45, 56, 61

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%% Two component channels pm %%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


%z1
%pm(31,30)
prop(1)=(z0+z1*(y(9)+y(10)+y(19)+y(20)))*y(30);

%z2
%pm(30,31)
prop(2)=z2*y(31);

%m1 catalysed by y(31)
%pm(33,32)
prop(3)=2*m1*y(31)*y(32);

%m2
%pm(32,33)   
prop(4)=m2*y(33);

%e1 catalysed by y(34)
%pm(36,35)
prop(5)=2*e1*y(34)*y(35);

%e2
%pm(35,36)  
prop(6)=e2*y(36)/sp;    %FAST



%ly2
%pm(5,3)
prop(7)=ly2*y(3);     

%pm(6,4)
prop(8)=ly2*y(4);

%pm(15,13)
prop(9)=ly2*y(13);

%pm(16,14)
prop(10)=ly2*y(14);

%tp
%pm(7,5)
prop(11)=tp*y(5);

%pm(8,6)
prop(12)=tp*y(6);

%pm(9,7)
prop(13)=tp*y(7);

%pm(10,8)
prop(14)=tp*y(8);

%pm(17,15)
prop(15)=tp*y(15);

%pm(18,16)
prop(16)=tp*y(16);

%pm(19,17)
prop(17)=tp*y(17);

%pm(20,18)
prop(18)=tp*y(18);

%ls1
%pm(3,4)
prop(19)=ls1*y(4);

%pm(5,6)
prop(20)=ls1*y(6);

%pm(7,8)
prop(21)=ls1*y(8);

%pm(9,10)
prop(22)=ls1*y(10);

%pm(13,14)
prop(23)=ls1*y(14);

%pm(15,16)
prop(24)=ls1*y(16);

%pm(17,18)
prop(25)=ls1*y(18);

%pm(19,20)
prop(26)=ls1*y(20);

%ls2,  catalysed by y(37) 
%pm(4,3)
prop(27)=ls2*y(37)*y(3);

%pm(6,5)
prop(28)=ls2*y(37)*y(5);

%pm(8,7)
prop(29)=ls2*y(37)*y(7);

%pm(10,9)
prop(30)=ls2*y(37)*y(9);

%pm(14,13)
prop(31)=ls2*y(37)*y(13);

%pm(16,15)
prop(32)=ls2*y(37)*y(15);

%pm(18,17)
prop(33)=ls2*y(37)*y(17);

%pm(20,19)
prop(34)=ls2*y(37)*y(19);

%m1 catalysed by y(31)
%pm(34,33)
prop(35)=m1*y(31)*y(33);

%m2
%pm(33,34)
prop(36)=2*m2*y(34);

%e1 catalysed by y(34)
%pm(37,36)
prop(37)=e1*y(34)*y(36)/sp;       %FAST

%e2
%pm(36,37)
prop(38)=2*e2*y(37)/sp;           %FAST

%s1
%pm(29,28)
prop(39)=(s0+s1*(y(5)+y(7)+y(9)+y(15)+y(17)+y(19)))*y(28)/sp;  %FAST  

%s2
%pm(28,29)
prop(40)=s2*y(29)/sp;                                 %FAST


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%% There component: synthesis pmm %%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%b1
%pmm(2,1,21)
prop(41)=b1*y(1)*y(21);  

%b2
%pmm(12,11,21)
prop(42)=b2*y(11)*y(21);

%lb
%pmm(3,2,27)
prop(43)=lb*y(27)*y(2);

%pmm(13,12,27)
prop(44)=lb*y(27)*y(12);

%ly1 
%pmm(22,21,29)
prop(45)=ly1*y(21)*y(29)/sp;    %FAST 

%pmm(23,2,29)
prop(46)=ly1*y(2)*y(29);

%pmm(24,3,29)
prop(47)=ly1*y(3)*y(29);

%pmm(24,5,29)
prop(48)=ly1*y(5)*y(29);

%pmm(24,7,29)
prop(49)=ly1*y(7)*y(29);

%pmm(24,9,29)
prop(50)=ly1*y(9)*y(29);

%pmm(25,12,29)
prop(51)=ly1*y(12)*y(29);

%pmm(26,13,29)
prop(52)=ly1*y(13)*y(29);

%pmm(26,15,29)
prop(53)=ly1*y(15)*y(29);

%pmm(26,17,29)
prop(54)=ly1*y(17)*y(29);

%pmm(26,19,29)
prop(55)=ly1*y(19)*y(29);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%% There component: dissociation mpp %%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%s2
%mpp(22,21,28)
prop(56)=s2*y(22)/sp;                             %FAST

%mpp(23,2,28)
prop(57)=s2*y(23);

%mpp(25,12,28)
prop(58)=s2*y(25);

%mpp(24,3,28)
prop(59)=s2*y(24);

%mpp(26,13,28)
prop(60)=s2*y(26);

%s3
%mpp(22,21,29)
prop(61)=s3*y(22)/sp;                        %FAST

%mpp(23,2,29)
prop(62)=s3*y(23);

%mpp(25,12,29)
prop(63)=s3*y(25);

%mpp(24,3,29)
prop(64)=s3*y(24);

%mpp(26,13,29)
prop(65)=s3*y(26);

%d1
%mpp(2,1,21)
prop(66)=d1*y(2);

%mpp(12,11,21)
prop(67)=d2*y(12);

%mpp(23,1,22)
prop(68)=d1*y(23);

%mpp(25,11,22)
prop(69)=d2*y(25); 

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%% Four component dissociation %%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


%mppp(3,1,21,27)
prop(70)=d1*y(3);   

%mppp(4,1,21,27)
prop(71)=d1*y(4);

%mppp(5,1,21,27)
prop(72)=d1*y(5);

%mppp(6,1,21,27)
prop(73)=d1*y(6);

%mppp(7,1,21,27)
prop(74)=d1*y(7);

%mppp(8,1,21,27)
prop(75)=d1*y(8);

%mppp(9,1,21,27)
prop(76)=d1*y(9);

%mppp(10,1,21,27)
prop(77)=d1*y(10);

%mppp(13,11,21,27)
prop(78)=d2*y(13);

%mppp(14,11,21,27)
prop(79)=d2*y(14);

%mppp(15,11,21,27)
prop(80)=d2*y(15);

%mppp(16,11,21,27)
prop(81)=d2*y(16);

%mppp(17,11,21,27)
prop(82)=d2*y(17);

%mppp(18,11,21,27)
prop(83)=d2*y(18);

%mppp(19,11,21,27)
prop(84)=d2*y(19);

%mppp(20,11,21,27)
prop(85)=d2*y(20);

%mppp(24,1,27,22)
prop(86)=d1*y(24);

%mppp(26,11,27,22)
prop(87)=d2*y(26);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

     ptot=sum(prop);
     r1=rand;                 
    
    tau=1/ptot*log(1/r1);     %time to next reaction
    tbiol=tbiol+tau;
    nstep=nstep+1;
    
    r2=rand*ptot;                      
    reaction=0;
    s=0;
                 

    while s<r2

        reaction=reaction+1;
        s=s+prop(reaction);
    end; 
    
    y=y+v(reaction,:);   %updating state vector




                if (tbiol>nn) 
                    if (nn<ttot+0.1)  
                      nn=nn+1;
                      yy(nn,:)=y;
                      T(nn)=tbiol;
                      if (nn==t0) 
                             y(1)=y(1)+pMHC11; 
                             y(11)=y(11)+pMHC22;
                         end;
                      tbiol
                      ii
                              
                    end;  
                end;   
    
%%%%%%%%%% OUTPUT  %%%%%%%%%%%
end; % end of single cell loop

zz=zz+yy;  

nstep
           
xxx(ii,:,:)=yy(:,:);  % data for all cells 
           
 


endtime=clock; 
Ttotsim=etime(endtime,starttime)

end    %end of the main loop
zz=zz/NNN;   % average
save data1   % saves all variables